package com.simonstl.moe;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.Map;
import com.simonstl.moe.explicit.*;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The Namespace class provides a foundation for MOE objects to store namespaces.</p>
*
* <p>There are some real complications in dealing with namespaces in an event context, 
* made only more difficult by the potential creation of partial trees.
* The SAX event startPrefixMapping() takes place before the startElement() call,
* and similarly endPrefixMapping() is reported after endElement().</p>
*
* <p>SAX also provides options for reporting/not reporting the attributes which 
* define namespaces.  In MOE, when events are assembled, the namespaces will be 
* associated with the elements containing the definitions.</p>
*
* <p>I've defined the Namespace class to be something of a broken attribute, much like 
* Namespace declarations in XML 1.0.  The prefix of the attribute is fixed to xmlns, 
* the localName defines the prefix defined by the namespace.  The toString() method
* handles the xmlns/xmlns:prefix issues.</p>
*
* <p>The namespace URI is stored as textual content, just as with attributes.  This
* potentially supports some rather perverse things like the use of general entities
* in the attribute values used to define namespaces.</p>
*
* <p>nsURI is fixed to <code>http://www.w3.org/TR/REC-xml-names/</code>.</p>
*

* <p>version 0.01 is the initial release and contains plenty of bogus placeholder functionality.</p>
*
* @version 0.01 25 August 2001
* @author Simon St.Laurent
**/

public class Namespace extends Attribute {

/**level is used by {@link com.simonstl.moe.namespace.NamespaceContext NamespaceContext} to determine where in a tree a given namespace applies.
**/

protected int level;
/**current indicates that a namespace needs to be declared in the current context.  If events came through a SAX filter stripping xmlns attributes, this is useful.
**/

protected boolean current;

/**
Override regular name object with NamespaceName.
**/
protected NamespaceName name;


public void setNsURI(String _nsURI) {
   if (name==null) {name=new NamespaceName();}
   name.setNsURI(_nsURI);
}

public String getNsURI() {
   if (name==null) {name=new NamespaceName();}

   return name.getNsURI();
}

public void setLocalName(String _localName){
   if (name==null) {name=new NamespaceName();}

   name.setLocalName(_localName);
}

public String getLocalName(){
   if (name==null) {name=new NamespaceName();}

   return name.getLocalName();
}

public void setPrefix(String _prefix){
   if (name==null) {name=new NamespaceName();}

   name.setPrefix(_prefix);
}

public String getPrefix(){
   if (name==null) {name=new NamespaceName();}

   return name.getPrefix();
}

/**
* Returns the Qualified Name (QName) for the component.
**/
public String getQName(){
   if (name==null) {name=new NamespaceName();}

   return name.getQName();
}

/**
* Sets the Qualified Name (QName) for the component.  Overrides current local name and prefix, if any.
**/
public void setQName(String QName){
   if (name==null) {name=new NamespaceName();}
   name.setQName(QName);
}

/**
* Returns the name object for the component.
**/
public NamingI getName(){
  return name;
}

/**
* Sets the name object for the component. 
**/
public void setName(NamingI _name){
   name=(NamespaceName) _name;
}



public String toString() {
   String returnVal=this.getQName();
   returnVal=returnVal+"=\"";
   returnVal=returnVal+getTextContent()+"\"";
   return returnVal;
}
public String toString(ComponentSetI namespaceContext){
  return "Namespace.toString(ComponentSetI namespaceContext) not yet implemented.";
}


public Object clone() {
   CoreComponentI newComponent=new Namespace();
   if (getName()!=null) {
     NamingI newName=(NamingI) this.getName().clone();
     newComponent.setName(newName);
   }
   if (getContent()!=null) {
     ComponentListI newContent=(ComponentListI) this.getContent().clone();
     newContent.setParent(newComponent);
     newComponent.setContent(newContent);
   }
   if (getUnorderedContent()!=null) {
     ComponentSetI newUnorderedContent=(ComponentSetI) this.getUnorderedContent().clone();
     newUnorderedContent.setParent(newComponent);
     newComponent.setUnorderedContent(newUnorderedContent);
   }
   if (getUnorderedContent()!=null) {
     AnnotableI thisAnnotations=(Annotations) this.getAllAnnotations();
     AnnotableI newAnnotations=(Annotations) thisAnnotations.clone();

     newComponent.addAll((Map) newAnnotations);
   }
   newComponent.setIsComplete(this.getIsComplete());

   return (Object) newComponent;
}


/**
* Namespace has the basic type NAMESPACE_DECL.
**/
public int getBasicType() {
   return NAMESPACE_DECL;
}


/**
* For now, the namespace class doesn't listen.  This may be modified to support more attribute-like functionality, but the issue of using general entities in namespaces gets complex rapidly.
**/
public boolean listen (CoreComponentI component){
  return false;
}

/**
* getLevel()is used by the {@link com.simonstl.moe.namespace.NamespaceContext NamespaceContext} object to figure out which declarations appeared where.
**/
public int getLevel(){
   return level;
}
/**
* setLevel()is used by the {@link com.simonstl.moe.namespace.NamespaceContext NamespaceContext} object to establish which declarations appeared where.
**/
public void setLevel(int _level){
   level=_level;
}

/**
* getCurrent()is used by the {@link com.simonstl.moe.explicit.Element Element} object to figure out if a namespace needs to be declared here.
**/
public boolean getCurrent(){
   return current;
}
/**
* setLevel()is used by the {@link com.simonstl.moe.namespace.NamespaceContext NamespaceContext} object to establish which declarations appeared in the current element.
**/
public void setCurrent(boolean _current){
   current=_current;
}

public static void main (String[] args) throws Exception {
    int numArgs=args.length;
    Namespace namespace=new Namespace();

 if (numArgs<1) {
    System.out.println("need arguments!");
 } else { 
   namespace.setContent(args[0]);
  if (numArgs>1) {
   namespace.setLocalName(args[1]);
 }
   namespace.print();
 }//end else

}

}