package com.simonstl.moe.factory;

import com.simonstl.moe.*;
import com.simonstl.moe.explicit.*;
import java.lang.ClassNotFoundException;
import java.lang.IllegalAccessException;
import java.lang.InstantiationException;
import java.lang.SecurityException;
import java.lang.ClassCastException;
import java.util.StringTokenizer;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The ComponentFactory class provides a relatively simple mechanism for creating new MOE components.  I've kept the constructors on the components deliberately minimal, preferring to use factory classes to create particular objects</p>
*
* <p>The ComponentFactory class uses System properties to define which classes map to which kinds of components.  More sophisticated classes (which I haven't written) could permit, for example, the mapping of particular MOE objects to classes based on things like the namespace URI and/or local name.</p>
*
<p>This class uses class information from the system properties:</p>
<pre>com.simonstl.moe.factory.element
com.simonstl.moe.factory.endElement
com.simonstl.moe.factory.attribute
com.simonstl.moe.factory.endAttribute
com.simonstl.moe.factory.characters
com.simonstl.moe.factory.cdataSect
com.simonstl.moe.factory.endCdataSect
com.simonstl.moe.factory.procInst
com.simonstl.moe.factory.endProcInst
com.simonstl.moe.factory.comment
com.simonstl.moe.factory.endComment
com.simonstl.moe.factory.namespace
com.simonstl.moe.factory.endNamespace
com.simonstl.moe.factory.whitespace</pre>
<p>If these properties are empty, they default to the classes in the {@link com.simonstl.moe com.simonstl.moe} package.
*
* @version 0.01 1 September 2001
* @author Simon St.Laurent
*/



public class ComponentFactory {

protected static String elementClass;
protected static String endElementClass;
protected static String attributeClass;
protected static String endAttributeClass;
protected static String charactersClass;
protected static String cdataSectClass;
protected static String endCdataSectClass;
protected static String procInstClass;
protected static String endProcInstClass;
protected static String commentClass;
protected static String endCommentClass;
protected static String namespaceClass;
protected static String endNamespaceClass;
protected static String whitespaceClass;

/**
Divides a QName into two pieces.
**/
public static String[] breakQName(String QName){
   String[] returnVal=new String[2];
   StringTokenizer parts=new StringTokenizer(QName,":");
   if (parts.countTokens()==1){
     returnVal[0]=null;
     returnVal[1]=QName;
   }
   if (parts.countTokens()==2){
     returnVal[0]=parts.nextToken();
     returnVal[1]=parts.nextToken();
   }
  return returnVal;
}


/**
Creates an empty element with a namespace URI, local name, and prefix.
**/
public static CoreComponentI element(String _nsURI, String _localName, String _prefix) throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.element"))==null) {
     System.setProperty("com.simonstl.moe.factory.element",
        "com.simonstl.moe.explicit.Element");
   }
   elementClass=System.getProperty("com.simonstl.moe.factory.element");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(elementClass);
   } catch (MOEException e) {
     throw e;
   } 
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates a complete element with a namespace URI, local name, prefix, and a text node.
**/

public static CoreComponentI element(String _nsURI, String _localName, String _prefix, String _text) throws MOEException  {
   Element returnVal=(Element) element(_nsURI, _localName, _prefix);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an element with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static CoreComponentI element(String _nsURI, String _localName, String _prefix, String _text, boolean _complete) throws MOEException  {
   Element returnVal=(Element) element(_nsURI, _localName, _prefix, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static CoreComponentI attribute(String _nsURI, String _localName, String _prefix) throws MOEException  {
   if ((System.getProperty("com.simonstl.moe.factory.attribute"))==null) {
     System.setProperty("com.simonstl.moe.factory.attribute",
        "com.simonstl.moe.explicit.Attribute");
   }

   attributeClass=System.getProperty("com.simonstl.moe.factory.attribute");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(attributeClass);
   } catch (MOEException e) {
     throw e;
   } 
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node.
**/
public static CoreComponentI attribute(String _nsURI, String _localName, String _prefix, String _text)  throws MOEException {
   Attribute returnVal=(Attribute) attribute(_nsURI, _localName, _prefix);
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates an attribute with a namespace URI, local name, prefix, and a text node, and allows the caller to specify whether or not the element is complete.
**/

public static CoreComponentI attribute(String _nsURI, String _localName, String _prefix, String _text, boolean _complete)  throws MOEException {
   Attribute returnVal=(Attribute) attribute(_nsURI, _localName, _prefix, _text);
   returnVal.setIsComplete(_complete);
   return returnVal;
}


/**
Creates a text node with textual content.
**/
public static CoreComponentI characters(String _text) throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.characters"))==null) {
     System.setProperty("com.simonstl.moe.factory.characters",
        "com.simonstl.moe.explicit.Characters");
   }
   charactersClass=System.getProperty("com.simonstl.moe.factory.characters");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(charactersClass);
   } catch (MOEException e) {
     throw e;
   } 
    returnVal.setContent(_text);
    return returnVal;
}

/**
Creates a complete Processing Instruction with target and textual content.
**/
public static CoreComponentI procInst(String _target, String _text) throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.procInst"))==null) {
     System.setProperty("com.simonstl.moe.factory.procInst",
        "com.simonstl.moe.explicit.ProcInst");
   }
   procInstClass=System.getProperty("com.simonstl.moe.factory.procInst");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(procInstClass);
   } catch (MOEException e) {
     throw e;
   } 
    returnVal.setLocalName(_target);
    returnVal.setContent(_text);
    returnVal.setIsComplete(true);
    return returnVal;
}

/**
Creates a Processing Instruction with target and textual content, allowing the code to specify whether or not it is complete.
**/
public CoreComponentI procInst(String _target, String _text, boolean _complete) throws MOEException {
    ProcInst returnVal=(ProcInst) procInst(_target, _text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}

/**
Creates a complete Comment with textual content.
**/
public static CoreComponentI comment(String _text) throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.comment"))==null) {
     System.setProperty("com.simonstl.moe.factory.comment",
        "com.simonstl.moe.explicit.Comment");
   }
   commentClass=System.getProperty("com.simonstl.moe.factory.comment");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(commentClass);
   } catch (MOEException e) {
     throw e;
   } 
    returnVal.setContent(_text);
    returnVal.setIsComplete(true);
    return returnVal;
}

/**
Creates a comment with textual content, allowing the code to specify whether or not it is complete.
**/
public static CoreComponentI comment(String _text, boolean _complete) throws MOEException {

    Comment returnVal=(Comment) comment(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}



/**
Creates a complete CDATA section with textual content.
**/
public static CoreComponentI cdataSect(String _text) throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.cdataSect"))==null) {
     System.setProperty("com.simonstl.moe.factory.cdataSect",
        "com.simonstl.moe.explicit.CDATASect");
   }
   cdataSectClass=System.getProperty("com.simonstl.moe.factory.cdataSect");
   CoreComponentI returnVal;
   try {
    returnVal=createComponent(cdataSectClass);
    } catch (MOEException e) {
      throw e;
    } 
   returnVal.setContent(_text);
   returnVal.setIsComplete(true);
   return returnVal;
}

/**
Creates a CDATA section with textual content, allowing the code to specify whether or not it is complete.
**/
public static CoreComponentI cdataSect(String _text, boolean _complete) throws MOEException {

    CDATASect returnVal=(CDATASect) cdataSect(_text);
    returnVal.setIsComplete(_complete);
    return returnVal;
}


/**
Creates a whitespace node with content.
**/
public static CoreComponentI whitespace(String _text) throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.whitespace"))==null) {
     System.setProperty("com.simonstl.moe.factory.whitespace",
        "com.simonstl.moe.explicit.Whitespace");
   }
   whitespaceClass=System.getProperty("com.simonstl.moe.factory.whitespace");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(whitespaceClass);
   } catch (MOEException e) {
     throw e;
   } 
    returnVal.setContent(_text);
    return returnVal;
}

/**
Creates a namespace.
**/
public static CoreComponentI namespace(String _nsURI, String _prefix)  throws MOEException {
  if ((System.getProperty("com.simonstl.moe.factory.namespace"))==null) {
     System.setProperty("com.simonstl.moe.factory.namespace",
        "com.simonstl.moe.explicit.Namespace");
  }
  namespaceClass=System.getProperty("com.simonstl.moe.factory.namespace");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(namespaceClass);
   } catch (MOEException e) {
     throw e;
   } 
   returnVal.setLocalName(_prefix);
   returnVal.setContent(_nsURI);
   return returnVal;
}

/**
Creates a default.
**/
public static CoreComponentI namespace(String _nsURI)  throws MOEException {
   return namespace(_nsURI, null);
}

/**
Creates an end element with a namespace URI, local name, and prefix.
**/
public static CoreComponentI endElement(String _nsURI, String _localName, String _prefix)  throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.endElement"))==null) {
     System.setProperty("com.simonstl.moe.factory.endElement",
        "com.simonstl.moe.explicit.EndElement");
   }
   endElementClass=System.getProperty("com.simonstl.moe.factory.endElement");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(endElementClass);
   } catch (MOEException e) {
     throw e;
   } 
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an empty attribute with a namespace URI, local name, and prefix.
**/
public static CoreComponentI endAttribute(String _nsURI, String _localName, String _prefix)  throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.endAttribute"))==null) {
     System.setProperty("com.simonstl.moe.factory.endAttribute",
        "com.simonstl.moe.explicit.EndAttribute");
   }
   endAttributeClass=System.getProperty("com.simonstl.moe.factory.endAttribute");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(endAttributeClass);
   } catch (MOEException e) {
     throw e;
   } 
   returnVal.setNsURI(_nsURI);
   returnVal.setLocalName(_localName);
   returnVal.setPrefix(_prefix);
   return returnVal;
}

/**
Creates an end to a namespace.
**/
public static CoreComponentI endNamespace(String _nsURI, String _prefix)  throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.endNamespace"))==null) {
     System.setProperty("com.simonstl.moe.factory.endNamespace",
        "com.simonstl.moe.explicit.EndNamespace");
   }
   endNamespaceClass=System.getProperty("com.simonstl.moe.factory.endNamespace");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(endNamespaceClass);
   } catch (MOEException e) {
     throw e;
   } 
   returnVal.setLocalName(_prefix);
   returnVal.setContent(_nsURI);
   return returnVal;
}



/**
Creates an end to a CDATA section.
**/
public static CoreComponentI endCdataSect() throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.endCdataSect"))==null) {
     System.setProperty("com.simonstl.moe.factory.endCdataSect",
        "com.simonstl.moe.explicit.EndCDATASect");
   }
   endCdataSectClass=System.getProperty("com.simonstl.moe.factory.endCdataSect");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(endCdataSectClass);
   } catch (MOEException e) {
     throw e;
   } 
   return returnVal;
}

public static CoreComponentI endProcInst() throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.endProcInst"))==null) {
     System.setProperty("com.simonstl.moe.factory.endProcInst",
        "com.simonstl.moe.explicit.EndProcInst");
   }
   endProcInstClass=System.getProperty("com.simonstl.moe.factory.endProcInst");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(endProcInstClass);
   } catch (MOEException e) {
     throw e;
   } 
   return returnVal;
}

public static CoreComponentI endComment() throws MOEException {
   if ((System.getProperty("com.simonstl.moe.factory.endComment"))==null) {
     System.setProperty("com.simonstl.moe.factory.endComment",
        "com.simonstl.moe.explicit.EndComment");
   }
   endCommentClass=System.getProperty("com.simonstl.moe.factory.endComment");

  CoreComponentI returnVal;
  try {
   returnVal=createComponent(endCommentClass);
   } catch (MOEException e) {
     throw e;
   } 
   return returnVal;
}

//thanks Leigh Dodds for much smaller version of this
protected static CoreComponentI createComponent (String className)
	throws MOEException {
	try {
	    return (CoreComponentI)(Class.forName(className).newInstance());
	} catch (Exception e1) {

		throw new MOEException("Can't create MOE Component", e1);

	}
}

public static void main(String[] args) throws Exception {
  ComponentFactory cf=new ComponentFactory();
  System.out.println("Incomplete element, no text.");
  Element test1=(Element) cf.element("http://simonstl.com/ns/test", "test", "prefix");
  test1.print();

  System.out.println("Incomplete element, no text.");
  Element test2=(Element) cf.element("http://simonstl.com/ns/test", "test", "prefix", "textual content", false);
  test2.print();

  System.out.println("Complete element, no text.");
  Element test3=(Element) cf.element("http://simonstl.com/ns/test", "test", "prefix", "textual content");
  test3.print();

  System.out.println("Attribute, no text.");
  Attribute test4=(Attribute) cf.attribute("http://simonstl.com/ns/test", "testAtt1", "prefix");
  test4.print();

  System.out.println("Attribute, with text.");
  Attribute test5=(Attribute) cf.attribute("http://simonstl.com/ns/test", "testAtt2", "prefix", "texties");
  test5.print();

  System.out.println("Attributes added to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("Attributes added repeatedly to complete element.");
  test3.addUnorderedContent(test4);
  test3.addUnorderedContent(test5);
  test3.print();

  System.out.println("CDATA section with text, incomplete.");
  CDATASect test6=(CDATASect) cf.cdataSect("weird content, ><", false);
  test6.print();

  System.out.println("CDATA section with text, complete.");
  CDATASect test7=(CDATASect) cf.cdataSect("weird content, ><");
  test7.print();

  System.out.println("Characters with text");
  Characters test8=(Characters) cf.characters("some content");
  test8.print();

  System.out.println("Namespace with prefix");
  Namespace test9=(Namespace) cf.namespace("http://simonstl.com", "goon");
  test9.print();

  System.out.println("Namespace without prefix");
  Namespace test10=(Namespace) cf.namespace("http://simonstl.com");
  test10.print();

}
}